#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>


#include "config.h"
#include "imapfilter.h"


int sock;


/*
 * Connect to mail server.
 */
int init_connection(char *serv, unsigned int port)
{
    struct sockaddr_in sa;
    struct hostent *he;

    memset((char *) &sa, 0, sizeof(struct sockaddr_in));

    sock = socket(PF_INET, SOCK_STREAM, 0);

    if (sock < 0) {
	error("imapfilter: create socket; %s\n", strerror(errno));
	return ERROR_NETWORK;
    }
    if (!(he = gethostbyname(serv))) {
	error("imapfilter: get network host entry; %s\n", strerror(errno));
	close_connection();
	return ERROR_NETWORK;
    }
    sa.sin_family = AF_INET;
    sa.sin_port = htons(port);
    sa.sin_addr = *(struct in_addr *) he->h_addr;

    if (connect(sock, (struct sockaddr *) & sa, sizeof(struct sockaddr))) {
	error("imapfilter: initiating connection; %s\n", strerror(errno));
	close_connection();
	return ERROR_NETWORK;
    }
    strncpy(serv, he->h_name, SERVER_LEN - 1);

    info("Connected to %s.\n", serv);
    log_info(LOG_SERVER, serv);

    return 0;
}


/*
 * Disconnect from mail server.
 */
int close_connection(void)
{
    if (close(sock)) {
	error("imapfilter: closing socket; %s\n", strerror(errno));
	return ERROR_NETWORK;
    } else
	return 0;
}
