#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include "config.h"
#include "imapfilter.h"
#include "data.h"


extern int sock;

static unsigned int tag = 0xF00000;	/* Every IMAP command is prefixed
					   with a unique [:alnum:] string. */

/*
 * Send to server data; a command.
 */
unsigned int send_command(char *cmd)
{

#ifdef DEBUG
    printf("debug: sending command: %s", cmd);
#endif

    if (write(sock, cmd, strlen(cmd)) == -1)
	fatal(ERROR_NETWORK, "imapfilter: error while sending command; %s",
	      strerror(errno));

    return tag++;
}


#ifdef DEBUG
/*
 * IMAP NOOP: does nothing always succeeds.
 */
int imap_noop(void)
{
    char cmd[SMALL_CMD];

    verbose("Client request: NOOP\n");

    snprintf(cmd, SMALL_CMD, "%X NOOP\r\n", tag);

    return send_command(cmd);
}
#endif


/*
 * IMAP CAPABILITY: requests listing of capabilities that the server supports
 */
int imap_capability(void)
{
    char cmd[SMALL_CMD];

    verbose("Client request: CAPABILITY\n");

    snprintf(cmd, SMALL_CMD, "%X CAPABILITY\r\n", tag);

    return send_command(cmd);
}


/*
 * IMAP LOGOUT: informs server that client is done.
 */
int imap_logout(void)
{
    char cmd[SMALL_CMD];

    verbose("Client request: LOGOUT\n");

    snprintf(cmd, SMALL_CMD, "%X LOGOUT\r\n", tag);

    return send_command(cmd);
}


/*
 * IMAP LOGIN: identifies client to server.
 */
int imap_login(char *user, char *pass)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: LOGIN\n");

    snprintf(cmd, MEDIUM_CMD, "%X LOGIN \"%s\" \"%s\"\r\n", tag, user,
	     pass);

    return send_command(cmd);
}


/*
 * IMAP EXAMINE: access a mailbox in READ-ONLY mode.
 *
int imap_examine(char *mbox)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: EXAMINE %s\n", mbox);

    snprintf(cmd, MEDIUM_CMD, "%X EXAMINE \"%s\"\r\n", tag, mbox);

    return send_command(cmd);
}*/


/*
 * IMAP SELECT: access a mailbox in READ-WRITE mode.
 */
int imap_select(char *mbox)
{
    char cmd[SMALL_CMD];

    verbose("Client request: SELECT\n");

    snprintf(cmd, SMALL_CMD, "%X SELECT \"%s\"\r\n", tag, mbox);

    return send_command(cmd);
}


/*
 * IMAP STATUS: requests status of the indicated mailbox.
 */
int imap_status(char *mbox, char *items)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: STATUS\n");

    snprintf(cmd, MEDIUM_CMD, "%X STATUS %s (%s)\r\n", tag, mbox, items);

    return send_command(cmd);
}


/*
 * IMAP CREATE: create mailbox.
 */
int imap_create(char *mbox)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: CREATE\n");

    snprintf(cmd, MEDIUM_CMD, "%X CREATE %s\r\n", tag, mbox);

    return send_command(cmd);
}



/*
 * IMAP SEARCH: searches the mailbox for messages that match certain criteria.
 */
int imap_search(char *search)
{
    char cmd[BIG_CMD];

    verbose("Client request: SEARCH\n");

    snprintf(cmd, BIG_CMD, "%X SEARCH %s\r\n", tag, search);

    return send_command(cmd);
}


/*
 * IMAP FETCH: retrieves data associated with a message.
 */
int imap_fetch(char *mesg, char *headers, int peek)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: FETCH\n");

    snprintf(cmd, MEDIUM_CMD,
	     "%X FETCH %s BODY%s[HEADER.FIELDS (%s)]\r\n", tag, mesg,
	     (peek ? ".PEEK" : ""), headers);

    return send_command(cmd);
}


/*
 * IMAP STORE: alters data associated with a message.
 */
int imap_store(char *mesg, char *flags)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: STORE\n");

    snprintf(cmd, MEDIUM_CMD, "%X STORE %s +FLAGS.SILENT (%s)\r\n", tag, mesg,
	     flags);

    return send_command(cmd);
}


/*
 * IMAP COPY: copy messages to mailbox.
 */
int imap_copy(char *mesg, char *mbox)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: COPY\n");

    snprintf(cmd, SMALL_CMD, "%X COPY %s \"%s\"\r\n", tag, mesg, mbox);

    return send_command(cmd);
}


/*
 * IMAP CLOSE: delete messages and return to authenticated state.
 */
int imap_close(void)
{
    char cmd[SMALL_CMD];

    verbose("Client request: CLOSE\n");

    snprintf(cmd, SMALL_CMD, "%X CLOSE\r\n", tag);

    return send_command(cmd);
}



/*
 * IMAP EXPUNGE: permanently removes any messages with the \Deleted flag set.
 */
int imap_expunge(void)
{
    char cmd[SMALL_CMD];

    verbose("Client request: EXPUNGE\n");

    snprintf(cmd, SMALL_CMD, "%X EXPUNGE\r\n", tag);

    return send_command(cmd);
}
